Simple Exercise on State Transitions in REST
==========

This simple REST server provides the template for
implementing the core state transition
explained in [https://www.infoq.com/articles/webber-rest-workflow](Webber, Parastatidis, Robinson, 2008: How to GET a Cup of Coffee).

The provided code have shortcuts: Only focus is on ordering a coffee
(Story 1), which is working, while payment of it (Story 3) is only
supported by Java support classes, while the actual REST server implementation is left as an exercise.

The core learning objective is to show how creating a coffee order
resource (POST on /order) also creates a payment resources, and this
resources is then updated (PUT /payment/order/{id} or similar) with
credit card information in order to pay for the coffee.

This is a central workflow aspect of REST: resources are created and
manipulated to force state transitions in the underlying system. This
is the **Hypermedia As The Engine Of Application State (HATEOAS)**
principle in REST.

How to execute:
---

  * gradle coffeeshop: Start the coffee server on port 4567
    
  * No client is provided, instead use 'curl' to directly interact
      with resources as outlined in the user stories below.

User Stories:
---

Story 1: Order a 'latte' by specifying the 'drink' attribute 

    curl -i -X POST -d '{"drink":"latte"}' localhost:4567/order
  
The Location of the order will be returned as well as the object
itself which specifies the cost.

Noting the Location in the header for the resource id generated; to
review the order

    curl -i localhost:4567/order/100

Shows contents of order 100. Not that no hypermedia links are returned
or part of the order object. In our representation the statemachine
is instead hard coded into the design of the server.

Story 2: Not supported. Left as an exercise

Story 3: Payment is not supported, but the Payment.java class template
is provided. As hint for the exercise, you should be able to PUT on the
payment resource along the lines of the following curl:

    curl -i -X PUT -d '{"cardno":"1234", "amount":"3.00"}' localhost:4567/payment/order/100
  
If there is an order with the given id, then the order is payed which
can be verified with another GET on the drink order. If not, appropriate
status code should be returned.

Credits
---

Henrik Bærbak Christensen, University of Aarhus

And... Original case by Webber, Parastatidis, Robinson.


